@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/wait.h>
+#include <string.h>
 #include "kaem.h"
 
 /* Prototypes from other files */
@@ -83,7 +84,7 @@ char* find_executable(char* name)
 	char* trial = calloc(MAX_STRING, sizeof(char));
 	char* MPATH = calloc(MAX_STRING, sizeof(char)); /* Modified PATH */
 	require(MPATH != NULL, "Memory initialization of MPATH in find_executable failed\n");
-	copy_string(MPATH, PATH);
+	strcpy(MPATH, PATH);
 	FILE* t;
 	char* next = find_char(MPATH, ':');
 	int index;
@@ -94,7 +95,7 @@ char* find_executable(char* name)
 	while(NULL != next)
 	{
 		/* Reset trial */
-		trial_length = string_length(trial);
+		trial_length = strlen(trial);
 		for(index = 0; index < trial_length; index = index + 1)
 		{
 			trial[index] = 0;
@@ -103,15 +104,15 @@ char* find_executable(char* name)
 		next[0] = 0;
 
 		/* prepend_string(MPATH, prepend_string("/", name)) */
-		mpath_length = string_length(MPATH);
+		mpath_length = strlen(MPATH);
 		for(index = 0; index < mpath_length; index = index + 1)
 		{
 			require(MAX_STRING > index, "Element of PATH is too long\n");
 			trial[index] = MPATH[index];
 		}
 		trial[index] = '/';
-		offset = string_length(trial);
-		name_length = string_length(name);
+		offset = strlen(trial);
+		name_length = strlen(name);
 		for(index = 0; index < name_length; index = index + 1)
 		{
 			require(MAX_STRING > index, "Element of PATH is too long\n");
@@ -119,7 +120,7 @@ char* find_executable(char* name)
 		}
 
 		/* Try the trial */
-		require(string_length(trial) < MAX_STRING, "COMMAND TOO LONG!\nABORTING HARD\n");
+		require(strlen(trial) < MAX_STRING, "COMMAND TOO LONG!\nABORTING HARD\n");
 		t = fopen(trial, "r");
 		if(NULL != t)
 		{
@@ -136,6 +137,7 @@ char* find_executable(char* name)
 /* Function to convert a Token linked-list into an array of strings */
 char** list_to_array(struct Token* s)
 {
+	char* hold;
 	struct Token* n;
 	n = s;
 	char** array = calloc(MAX_ARRAY, sizeof(char*));
@@ -161,7 +163,7 @@ char** list_to_array(struct Token* s)
 		else
 		{ /* It is a var */
 			/* prepend_string(n->var, prepend_string("=", n->value)) */
-			var_length = string_length(n->var);
+			var_length = strlen(n->var);
 			for(i = 0; i < var_length; i = i + 1)
 			{
 				element[i] = n->var[i];
@@ -169,13 +171,15 @@ char** list_to_array(struct Token* s)
 			element[i] = '=';
 			i = i + 1;
 			offset = i;
-			value_length = string_length(n->value);
+			value_length = strlen(n->value);
 			for(i = 0; i < value_length; i = i + 1)
 			{
 				element[i + offset] = n->value[i];
 			}
 		}
-		copy_string(array[index], element);
+
+		/* Insert elements if not empty */
+		if(!match("", element)) strcpy(array[index], element);
 
 		n = n->next;
 		index = index + 1;
@@ -186,6 +190,31 @@ char** list_to_array(struct Token* s)
 	return array;
 }
 
+/* Function to handle the correct options for escapes */
+int handle_escape(int c)
+{
+	if(c == '\n')
+	{ /* Do nothing - eat up the newline */
+		return -1;
+	}
+	else if('n' == c)
+	{ /* Add a newline to the token */
+		return '\n';
+	}
+	else if('r' == c)
+	{ /* Add a return to the token */
+		return '\r';
+	}
+	else if('\\' == c)
+	{ /* Add a real backslash to the token */
+		return '\\';
+	}
+	else
+	{ /* Just add it to the token (eg, quotes) */
+		return c;
+	}
+}
+
 /*
  * TOKEN COLLECTION FUNCTIONS
  */
@@ -208,10 +237,11 @@ void collect_comment(FILE* input)
 }
 
 /* Function for collecting strings and removing the "" pair that goes with them */
-int collect_string(FILE* input, struct Token* n, int index)
+int collect_string(FILE* input, char* n, int index)
 {
 	int string_done = FALSE;
 	int c;
+	int cc;
 	do
 	{
 		/* Bounds check */
@@ -219,14 +249,21 @@ int collect_string(FILE* input, struct Token* n, int index)
 		c = fgetc(input);
 		require(EOF != c, "IMPROPERLY TERMINATED STRING!\nABORTING HARD\n");
 
-		if('"' == c)
+		if('\\' == c)
+		{ /* We are escaping the next character */
+			/* This correctly handles escaped quotes as it just returns the quote */
+			c = fgetc(input);
+			cc = handle_escape(c);
+			n[index] = c;
+			index = index + 1;
+		}
+		else if('"' == c)
 		{ /* End of string */
 			string_done = TRUE;
 		}
 		else
 		{
-			require(MAX_STRING > index, "LINE IS TOO LONG\nABORTING HARD\n");
-			n->value[index] = c;
+			n[index] = c;
 			index = index + 1;
 		}
 	} while(string_done == FALSE);
@@ -234,9 +271,10 @@ int collect_string(FILE* input, struct Token* n, int index)
 }
 
 /* Function to parse and assign token->value */
-int collect_token(FILE* input, struct Token* n, int last_index)
+int collect_token(FILE* input, char* n, int last_index)
 {
 	int c;
+	int cc;
 	int token_done = FALSE;
 	char* token = calloc(MAX_STRING, sizeof(char));
 	require(token != NULL, "Memory initialization of token in collect_token failed\n");
@@ -279,26 +317,14 @@ int collect_token(FILE* input, struct Token* n, int last_index)
 			if(0 == index) index = last_index;
 		}
 		else if('\\' == c)
-		{ /* Support for escapes; drops the char after */
-			/******************************************************************
-			 * This is a design decision; it is primarily made for newlines.  *
-			 * Unlike the way normal shells do escapes (making the next char  *
-			 * actually do something), this just eats it up. Why? Simply, to  *
-			 * aid formatting (mostly through newlines). Eventually, we will  *
-			 * make it do it the proper way... but for now it's staying like  *
-			 * this. Feel free to send in a patch if you have a solution!     *
-			 * Because of this, I have decided that since the behaviour is    *
-			 * signficant enough, when warnings are enabled, we will give a   *
-			 * warning about this.                                            *
-			 ******************************************************************/
+		{ /* Support for escapes */
 			c = fgetc(input); /* Skips over \, gets the next char */
-			if(WARNINGS && c != '\n')
-			{
-				file_print("WARNING: The character '", stdout);
-				fputc(c, stdout);
-				file_print("' just got eaten up because of an unsupported escape sequence; see kaem.c:collect_token for more information.\n", stdout);
+			cc = handle_escape(c);
+			if(-1 != cc)
+			{ /* We need to put it into the token */
+				n[index] = cc;
 			}
-			index = index + 2;
+			index = index + 1;
 		}
 		else if(0 == c)
 		{ /* We have come to the end of the token */
@@ -306,8 +332,7 @@ int collect_token(FILE* input, struct Token* n, int last_index)
 		}
 		else
 		{ /* It's a character to assign */
-			require(MAX_STRING > index, "LINE IS TOO LONG\nABORTING HARD\n");
-			n->value[index] = c;
+			n[index] = c;
 			index = index + 1;
 		}
 	} while (token_done == FALSE);
@@ -324,7 +349,7 @@ int collect_token(FILE* input, struct Token* n, int last_index)
 int is_envar(char* token)
 {
 	int i = 0;
-	int token_length = string_length(token);
+	int token_length = strlen(token);
 	while(i < token_length)
 	{
 		if(token[i] == '=')
@@ -337,63 +362,65 @@ int is_envar(char* token)
 }
 
 /* Add an envar */
-int add_envar()
+void add_envar()
 {
+	/* Pointers to strings we want */
+	char* name = calloc(strlen(token->value) + 4, sizeof(char));
+	char* value = token->value;
+	char* newvalue;
+	int i = 0;
+
+	/* Isolate the name */
+	while('=' != value[i])
+	{
+		name[i] = value[i];
+		i = i + 1;
+	}
+
+	/* Isolate the value */
+	newvalue = name + i + 2;
+	value = value + i + 1;
+	i = 0;
+	require(0 != value[i], "add_envar recieved improper variable\n");
+	while(0 != value[i])
+	{
+		newvalue[i] = value[i];
+		i = i + 1;
+	}
+
 	/* If we are in init-mode and this is the first var env == NULL, rectify */
 	if(env == NULL)
 	{
 		env = calloc(1, sizeof(struct Token));
 		require(env != NULL, "Memory initialization of env failed\n");
-	}
-
-	struct Token* n;
-	n = env;
-	/* Traverse to end of linked-list */
-	while(n->next != NULL)
-	{
-		if(n->next->value == NULL || n->next->var == NULL) break;
-		n = n->next;
-	}
-	/* Initialize new node */
-	/* See first comment, this situation means no new node */
-	if(n->value != NULL || n->var != NULL || n->next != NULL)
-	{
-		n->next = calloc(1, sizeof(struct Token));
-		require(n->next != NULL, "Memory initialization of next env node in add_envar failed\n");
-		n = n->next;
+		env->var = name; /* Add our first variable */
 	}
 
 	/*
-	 * If you are confused about n->* and token->value here:
-	 * token->value: is the token with the raw data. Part of token linked list.
-	 * n->*: is where it goes. Part of env linked list.
+	 * If the name of the envar is PATH, then we need to set our (internal)
+	 * global PATH value.
 	 */
-	/* Get n->var */
-	int index = 0;
-	n->var = calloc(MAX_STRING, sizeof(char));
-	require(n->var != NULL, "Memory initialization of n->var in add_envar failed\n");
-	int token_length = string_length(token->value);
-	/* Copy into n->var up to = */
-	while(token->value[index] != '=')
+	if(match(name, "PATH"))
 	{
-		if(index >= token_length) return TRUE;
-		n->var[index] = token->value[index];
-		index = index + 1;
+		strcpy(PATH, newvalue);
 	}
 
-	/* Get n->value */
-	index = index + 1; /* Skip over = */
-	int offset = index;
-	n->value = calloc(MAX_STRING, sizeof(char));
-	require(n->value != NULL, "Memory initialization of n->value in add_envar failed\n");
-	/* Copy into n->value up to end of token */
-	while(token->value[index] != 0)
+	struct Token* n = env;
+
+	/* Find match if possible */
+	while(!match(name, n->var))
 	{
-		if(index >= token_length) return TRUE;
-		n->value[index - offset] = token->value[index];
-		index = index + 1;
+		if(NULL == n->next)
+		{
+			n->next = calloc(1, sizeof(struct Token));
+			require(n->next != NULL, "Memory initialization of next env node in add_envar failed\n");
+			n->next->var = name;
+		} /* Loop will match and exit */
+		n = n->next;
 	}
-	return FALSE;
+
+	/* Since we found the variable we need only to set it to its new value */
+	n->value = newvalue;
 }
 
 /* cd builtin */
@@ -414,8 +441,8 @@ int pwd()
 	require(path != NULL, "Memory initialization of path in pwd failed\n");
 	getcwd(path, MAX_STRING);
 	require(!match("", path), "getcwd() failed\n");
-	file_print(path, stdout);
-	file_print("\n", stdout);
+	fputs(path, stdout);
+	fputs("\n", stdout);
 	return FALSE;
 }
 
@@ -430,20 +457,20 @@ int set()
 	char* options = calloc(MAX_STRING, sizeof(char));
 	require(options != NULL, "Memory initialization of options in set failed\n");
 
-	int last_position = string_length(token->value) - 1;
+	int last_position = strlen(token->value) - 1;
 	for(i = 0; i < last_position; i = i + 1)
 	{
 		options[i] = token->value[i + 1];
 	}
 	/* Parse the options */
-	int options_length = string_length(options);
+	int options_length = strlen(options);
 	for(i = 0; i < options_length; i = i + 1)
 	{
 		if(options[i] == 'a')
 		{ /* set -a is on by default and cannot be disabled at this time */
 			if(WARNINGS)
 			{
-				file_print("set -a is on by default and cannot be disabled\n", stdout);
+				fputs("set -a is on by default and cannot be disabled\n", stdout);
 			}
 			continue;
 		}
@@ -460,15 +487,15 @@ int set()
 			 * We don't do just -x because we support multiple options in one command,
 			 * eg set -ex.
 			 */
-			file_print(" +> set -", stdout);
-			file_print(options, stdout);
-			file_print("\n", stdout);
+			fputs(" +> set -", stdout);
+			fputs(options, stdout);
+			fputs("\n", stdout);
 			fflush(stdout);
 		}
 		else
 		{ /* Invalid */
 			fputc(options[i], stderr);
-			file_print(" is an invalid set option!\n", stderr);
+			fputs(" is an invalid set option!\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -482,12 +509,12 @@ void echo()
 {
 	if(token->next == NULL)
 	{ /* No arguments */
-		file_print("\n", stdout);
+		fputs("\n", stdout);
 		return;
 	}
 	if(token->next->value == NULL)
 	{ /* No arguments */
-		file_print("\n", stdout);
+		fputs("\n", stdout);
 		return;
 	}
 	token = token->next; /* Skip the actual echo */
@@ -495,11 +522,10 @@ void echo()
 	{ /* Output each argument to echo to stdout */
 		/* M2-Planet doesn't let us do this in the while */
 		if(token->value == NULL) break;
-		file_print(token->value, stdout);
-		file_print(" ", stdout);
+		fputs(token->value, stdout);
 		token = token->next;
 	}
-	file_print("\n", stdout);
+	fputs("\n", stdout);
 }
 
 /* unset builtin */
@@ -515,10 +541,8 @@ void unset()
 		/* Look for the variable; we operate on ->next because we need to remove ->next */
 		while(e->next != NULL)
 		{
-			if(match(e->next->var, t->value))
-			{
-				break;
-			}
+			if(NULL == t->value) break;
+			if(match(e->next->var, t->value)) break;
 			e = e->next;
 		}
 		t = t->next;
@@ -539,8 +563,7 @@ int execute()
 	/* Actually do the execution */
 	if(is_envar(token->value) == TRUE)
 	{
-		rc = add_envar();
-		if(STRICT) require(rc == FALSE, "Adding of an envar failed!\n");
+		add_envar();
 		return 0;
 	}
 	else if(match(token->value, "cd"))
@@ -583,9 +606,9 @@ int execute()
 	{
 		if(STRICT == TRUE)
 		{
-			file_print("WHILE EXECUTING ", stderr);
-			file_print(token->value, stderr);
-			file_print(" NOT FOUND!\nABORTING HARD\n", stderr);
+			fputs("WHILE EXECUTING ", stderr);
+			fputs(token->value, stderr);
+			fputs(" NOT FOUND!\nABORTING HARD\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 		/* If we are not strict simply return */
@@ -596,9 +619,9 @@ int execute()
 	/* Ensure fork succeeded */
 	if (f == -1)
 	{
-		file_print("WHILE EXECUTING ", stderr);
-		file_print(token->value, stderr);
-		file_print("fork() FAILED\nABORTING HARD\n", stderr);
+		fputs("WHILE EXECUTING ", stderr);
+		fputs(token->value, stderr);
+		fputs("fork() FAILED\nABORTING HARD\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 	else if (f == 0)
@@ -632,50 +655,61 @@ int collect_command(FILE* script, char** argv)
 {
 	command_done = FALSE;
 	/* Initialize token */
-	token = calloc(1, sizeof(struct Token));
-	require(token != NULL, "Memory initialization of token in collect_command failed\n");
 	struct Token* n;
-	n = token;
+	n = calloc(1, sizeof(struct Token));
+	require(n != NULL, "Memory initialization of token in collect_command failed\n");
+	char* s = calloc(MAX_STRING, sizeof(char));
+	require(s != NULL, "Memory initialization of token in collect_command failed\n");
+	token = n;
 	int index = 0;
+
 	/* Get the tokens */
 	while(command_done == FALSE)
 	{
-		n->value = calloc(MAX_STRING, sizeof(char));
-		index = collect_token(script, n, index);
+		index = collect_token(script, s, index);
 		/* Don't allocate another node if the current one yielded nothing, OR
 		 * if we are done.
 		 */
-		if((n->value != NULL && match(n->value, "") == FALSE) && command_done == FALSE)
+
+		if(match(s, "")) continue;
+
+		/* add to token */
+		n->value = s;
+		s = calloc(MAX_STRING, sizeof(char));
+		require(s != NULL, "Memory initialization of next token node in collect_command failed\n");
+
+		/* Deal with variables */
+		handle_variables(argv, n);
+
+		/* If the variable expands into nothing*/
+		if(match(n->value, " "))
 		{
-			n->next = calloc(MAX_STRING, sizeof(char));
-			require(token != NULL, "Memory initialization of next token node in collect_command failed\n");
-			n = n->next;
+			n->value = NULL;
+			continue;
 		}
+
+		/* Prepare for next loop */
+		n->next = calloc(1, sizeof(struct Token));
+		require(n->next != NULL, "Memory initialization of next token node in collect_command failed\n");
+		n = n->next;
 	}
 	/* -1 means the script is done */
 	if(EOF == index) return index;
 
-	n = token;
-	while(n != NULL)
-	{ /* Substitute variables into each token */
-		if(n->value == NULL) break;
-		handle_variables(argv, n);
-		/* Advance to next node */
-		n = n->next;
-	}
-
 	/* Output the command if verbose is set */
 	/* Also if there is nothing in the command skip over */
-	if(VERBOSE && match(token->value, "") == FALSE)
+	if(VERBOSE && !match(token->value, "") && !match(token->value, NULL))
 	{
 		n = token;
-		file_print(" +> ", stdout);
+		fputs(" +>", stdout);
 		while(n != NULL)
 		{ /* Print out each token token */
+			fputs(" ", stdout);
 			/* M2-Planet doesn't let us do this in the while */
-			if(n->value == NULL) break;
-			file_print(n->value, stdout);
-			file_print(" ", stdout);
+			if(n->value != NULL)
+			{
+				if(!match(n->value, "")) fputs(n->value, stdout);
+			}
 			n = n->next;
 		}
 		fputc('\n', stdout);
@@ -709,9 +743,9 @@ void run_script(FILE* script, char** argv)
 		status = execute();
 		if(STRICT == TRUE && (0 != status))
 		{ /* Clearly the script hit an issue that should never have happened */
-			file_print("Subprocess error ", stderr);
-			file_print(numerate_number(status), stderr);
-			file_print("\nABORTING HARD\n", stderr);
+			fputs("Subprocess error ", stderr);
+			fputs(int2str(status, 10, TRUE), stderr);
+			fputs("\nABORTING HARD\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -720,6 +754,13 @@ void run_script(FILE* script, char** argv)
 /* Function to populate env */
 void populate_env(char** envp)
 {
+	/* You can't populate a NULL environment */
+	if(NULL == envp) return;
+
+	/* avoid empty arrays */
+	int max = array_length(envp);
+	if(0 == max) return;
+
 	/* Initialize env and n */
 	env = calloc(1, sizeof(struct Token));
 	require(env != NULL, "Memory initialization of env failed\n");
@@ -730,7 +771,8 @@ void populate_env(char** envp)
 	int j;
 	int k;
 	char* envp_line;
-	for(i = 0; i < array_length(envp); i = i + 1)
+
+	for(i = 0; i < max; i = i + 1)
 	{
 		n->var = calloc(MAX_STRING, sizeof(char));
 		require(n->var != NULL, "Memory initialization of n->var in population of env failed\n");
@@ -745,7 +787,7 @@ void populate_env(char** envp)
 		 */
 		envp_line = calloc(MAX_STRING, sizeof(char));
 		require(envp_line != NULL, "Memory initialization of envp_line in population of env failed\n");
-		copy_string(envp_line, envp[i]);
+		strcpy(envp_line, envp[i]);
 		while(envp_line[j] != '=')
 		{ /* Copy over everything up to = to var */
 			n->var[j] = envp_line[j];
@@ -799,9 +841,9 @@ int main(int argc, char** argv, char** envp)
 		}
 		else if(match(argv[i], "-h") || match(argv[i], "--help"))
 		{ /* Help information */
-			file_print("Usage: ", stdout);
-			file_print(argv[0], stdout);
-			file_print(" [-h | --help] [-V | --version] [--file filename | -f filename] [-i | --init-mode] [-v | --verbose] [--strict] [--warn] [--fuzz]\n", stdout);
+			fputs("Usage: ", stdout);
+			fputs(argv[0], stdout);
+			fputs(" [-h | --help] [-V | --version] [--file filename | -f filename] [-i | --init-mode] [-v | --verbose] [--strict] [--warn] [--fuzz]\n", stdout);
 			exit(EXIT_SUCCESS);
 		}
 		else if(match(argv[i], "-f") || match(argv[i], "--file"))
@@ -819,7 +861,7 @@ int main(int argc, char** argv, char** envp)
 		}
 		else if(match(argv[i], "-V") || match(argv[i], "--version"))
 		{ /* Output version */
-			file_print("kaem version 1.1.0\n", stdout);
+			fputs("kaem version 1.1.0\n", stdout);
 			exit(EXIT_SUCCESS);
 		}
 		else if(match(argv[i], "-v") || match(argv[i], "--verbose"))
@@ -848,7 +890,7 @@ int main(int argc, char** argv, char** envp)
 		}
 		else
 		{ /* We don't know this argument */
-			file_print("UNKNOWN ARGUMENT\n", stdout);
+			fputs("UNKNOWN ARGUMENT\n", stdout);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -872,20 +914,23 @@ int main(int argc, char** argv, char** envp)
 	{ /* We didn't find either of PATH or USERNAME -- use a generic PATH */
 		PATH = calloc(MAX_STRING, sizeof(char));
 		require(PATH != NULL, "Memory initialization of PATH failed\n");
-		copy_string(PATH, "/root/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin");
+		strcpy(PATH, "/root/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin");
 	}
 	else if(NULL == PATH)
 	{ /* We did find a username but not a PATH -- use a generic PATH but with /home/USERNAME */
-		PATH = prepend_string("/home/", prepend_string(USERNAME,"/bin:/usr/local/bin:/usr/bin:/bin:/usr/local/games:/usr/games"));
+		PATH = calloc(MAX_STRING, sizeof(char));
+		PATH = strcat(PATH, "/home/");
+		PATH = strcat(PATH, USERNAME);
+		PATH = strcat(PATH, "/bin:/usr/local/bin:/usr/bin:/bin:/usr/local/games:/usr/games");
 	}
 
 	/* Open the script */
 	script = fopen(filename, "r");
 	if(NULL == script)
 	{
-		file_print("The file: ", stderr);
-		file_print(filename, stderr);
-		file_print(" can not be opened!\n", stderr);
+		fputs("The file: ", stderr);
+		fputs(filename, stderr);
+		fputs(" can not be opened!\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
