@@ -17,36 +17,41 @@
  * along with mescc-tools.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdio.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
-#include <getopt.h>
+#include "M2libc/bootstrappable.h"
 
-//CONSTANT max_string 4096
+/* Internal processing Constants */
+// CONSTANT max_string 4096
 #define max_string 4096
-//CONSTANT PROCESSED 1
+// CONSTANT PROCESSED 1
 #define PROCESSED 1
-//CONSTANT STR 2
+// CONSTANT STR 2
 #define STR 2
-//CONSTANT NEWLINE 3
+// CONSTANT NEWLINE 3
 #define NEWLINE 3
 
-//CONSTANT TRUE 1
-#define TRUE 1
-//CONSTANT FALSE 0
-#define FALSE 0
-
+/* Unique code for each architecture */
 // CONSTANT KNIGHT 0
 #define KNIGHT 0
-// CONSTANT X86 1
-#define X86 1
-// CONSTANT AMD64 2
-#define AMD64 2
+// CONSTANT X86 3
+#define X86 0x03
+// CONSTANT AMD64 62
+#define AMD64 0x3E
 // CONSTANT ARMV7L 40
-#define ARMV7L 40
-// CONSTANT AARM64 80
-#define AARM64 80
+#define ARMV7L 0x28
+// CONSTANT AARM64 183
+#define AARM64 0xB7
+// CONSTANT PPC64LE 21
+#define PPC64LE 0x15
+// CONSTANT RISCV32 243
+#define RISCV32 0xF3
+// CONSTANT RISCV64 65779
+#define RISCV64 0x100F3 /* Because RISC-V unlike all other architectures does get a seperate e_machine when changing from 32 to 64bit */
 
+
+/* How do you want that output? */
 // CONSTANT HEX 16
 #define HEX 16
 // CONSTANT OCTAL 8
@@ -54,16 +59,9 @@
 // CONSTANT BINARY 2
 #define BINARY 2
 
-
-/* Imported functions */
-char* numerate_number(int a);
-int hex2char(int c);
-int in_set(int c, char* s);
-int match(char* a, char* b);
-int numerate_string(char *a);
-int string_length(char* a);
-void file_print(char* s, FILE* f);
-void require(int bool, char* error);
+/* Imported from stringify.c */
+int stringify(char* s, int digits, int divisor, int value, int shift);
+void LittleEndian(char* start, int ByteMode);
 
 struct blob
 {
@@ -86,7 +84,6 @@ struct Token
 FILE* source_file;
 FILE* destination_file;
 int BigEndian;
-int BigBitEndian;
 int ByteMode;
 int Architecture;
 int linenumber;
@@ -100,10 +97,10 @@ struct blob** hash_table;
 
 void line_error(char* filename, int linenumber)
 {
-	file_print(filename, stderr);
-	file_print(":", stderr);
-	file_print(numerate_number(linenumber), stderr);
-	file_print(" :", stderr);
+	fputs(filename, stderr);
+	fputs(":", stderr);
+	fputs(int2str(linenumber,10, FALSE), stderr);
+	fputs(" :", stderr);
 }
 
 void ClearScratch()
@@ -153,9 +150,9 @@ void NewBlob(int size)
 {
 	blob_count = blob_count + 1;
 	struct blob* a = calloc(1, sizeof(struct blob));
-	require(NULL != a, "Exhusted available memory\n");
+	require(NULL != a, "Exhausted available memory\n");
 	a->Text = calloc(size + 1, sizeof(char));
-	require(NULL != a->Text, "Exhusted available memory\n");
+	require(NULL != a->Text, "Exhausted available memory\n");
 
 	int i = 0;
 	while(i <= size)
@@ -173,7 +170,7 @@ struct Token* newToken(char* filename, int linenumber)
 	struct Token* p;
 
 	p = calloc (1, sizeof (struct Token));
-	require(NULL != p, "Exhusted available memory\n");
+	require(NULL != p, "Exhausted available memory\n");
 
 	p->filename = filename;
 	p->linenumber = linenumber;
@@ -231,7 +228,13 @@ struct Token* store_atom(struct Token* head, char c, char* filename)
 		SCRATCH[i] = ch;
 		ch = fgetc(source_file);
 		i = i + 1;
-		require(i < max_string, "storing atom of size larger than max_string\n");
+		if(i >= max_string)
+		{
+			fputs("storing atom of size larger than max_string\n", stderr);
+			line_error(filename, linenumber);
+			fputc('\n', stderr);
+			exit(EXIT_FAILURE);
+		}
 		if(EOF == ch) break;
 	} while (!in_set(ch, "\t\n "));
 
@@ -266,9 +269,9 @@ struct blob* store_string(char c, char* filename)
 		if(max_string == i)
 		{
 			line_error(filename, linenumber);
-			file_print("String: ", stderr);
-			file_print(SCRATCH, stderr);
-			file_print(" exceeds max string size\n", stderr);
+			fputs("String: ", stderr);
+			fputs(SCRATCH, stderr);
+			fputs(" exceeds max string size\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 	} while(ch != c);
@@ -348,9 +351,9 @@ void line_macro(struct Token* p)
 			if(PROCESSED == i->next->contents->type)
 			{
 				line_error(i->filename, i->linenumber);
-				file_print("Multiple definitions for macro ", stderr);
-				file_print(i->next->contents->Text, stderr);
-				file_print("\n", stderr);
+				fputs("Multiple definitions for macro ", stderr);
+				fputs(i->next->contents->Text, stderr);
+				fputs("\n", stderr);
 				exit(EXIT_FAILURE);
 			}
 
@@ -372,7 +375,7 @@ void line_macro(struct Token* p)
 void hexify_string(struct blob* p)
 {
 	char* table = "0123456789ABCDEF";
-	int i = string_length(p->Text);
+	int i = strlen(p->Text);
 	int size;
 
 	if(HEX == ByteMode) size = (((i << 1) + i) + 12);
@@ -382,7 +385,7 @@ void hexify_string(struct blob* p)
 
 	require(1 != size, "hexify_string lacked a valid bytemode\n");
 	char* d = calloc(size, sizeof(char));
-	require(NULL != d, "Exhusted available memory\n");
+	require(NULL != d, "Exhausted available memory\n");
 	p->Expression = d;
 	char* S = p->Text;
 
@@ -466,7 +469,7 @@ char* pad_nulls(int size, char* nil)
 	else if (BINARY == ByteMode) size = size * 8;
 
 	char* s = calloc(size + 1, sizeof(char));
-	require(NULL != s, "Exhusted available memory\n");
+	require(NULL != s, "Exhausted available memory\n");
 
 	int i = 0;
 	while(i < size)
@@ -494,7 +497,7 @@ void preserve_other(struct blob* p)
 			}
 			else if('<' == c)
 			{
-				i->Expression = pad_nulls(numerate_string(i->Text + 1), i->Text);
+				i->Expression = pad_nulls(strtoint(i->Text + 1), i->Text);
 			}
 		}
 	}
@@ -504,11 +507,11 @@ void bound_values(int displacement, int number_of_bytes, int low, int high)
 {
 	if((high < displacement) || (displacement < low))
 	{
-		file_print("A displacement of ", stderr);
-		file_print(numerate_number(displacement), stderr);
-		file_print(" does not fit in ", stderr);
-		file_print(numerate_number(number_of_bytes), stderr);
-		file_print(" bytes\n", stderr);
+		fputs("A displacement of ", stderr);
+		fputs(int2str(displacement, 10, TRUE), stderr);
+		fputs(" does not fit in ", stderr);
+		fputs(int2str(number_of_bytes, 10, TRUE), stderr);
+		fputs(" bytes\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -532,77 +535,14 @@ void range_check(int displacement, int number_of_bytes)
 		return;
 	}
 
-	file_print("Received an invalid number of bytes in range_check\n", stderr);
+	fputs("Received an invalid number of bytes in range_check\n", stderr);
 	exit(EXIT_FAILURE);
 }
 
-void reverseBitOrder(char* c)
-{
-	if(NULL == c) return;
-	if(0 == c[1]) return;
-	int hold = c[0];
-
-	if(HEX == ByteMode)
-	{
-		c[0] = c[1];
-		c[1] = hold;
-		reverseBitOrder(c+2);
-	}
-	else if(OCTAL == ByteMode)
-	{
-		c[0] = c[2];
-		c[2] = hold;
-		reverseBitOrder(c+3);
-	}
-	else if(BINARY == ByteMode)
-	{
-		c[0] = c[7];
-		c[7] = hold;
-		hold = c[1];
-		c[1] = c[6];
-		c[6] = hold;
-		hold = c[2];
-		c[2] = c[5];
-		c[5] = hold;
-		hold = c[3];
-		c[3] = c[4];
-		c[4] = hold;
-		reverseBitOrder(c+8);
-	}
-}
-
-void LittleEndian(char* start)
-{
-	char* end = start;
-	char* c = start;
-	while(0 != end[0]) end = end + 1;
-	int hold;
-	for(end = end - 1; start < end; start = start + 1)
-	{
-		hold = start[0];
-		start[0] = end[0];
-		end[0] = hold;
-		end = end - 1;
-	}
-
-	if(BigBitEndian) reverseBitOrder(c);
-}
-
-int stringify(char* s, int digits, int divisor, int value, int shift)
-{
-	int i = value;
-	if(digits > 1)
-	{
-		i = stringify(s+1, (digits - 1), divisor, value, shift);
-	}
-	s[0] = hex2char(i & (divisor - 1));
-	return (i >> shift);
-}
-
 char* express_number(int value, char c)
 {
 	char* ch = calloc(42, sizeof(char));
-	require(NULL != ch, "Exhusted available memory\n");
+	require(NULL != ch, "Exhausted available memory\n");
 	int size;
 	int number_of_bytes;
 	int shift;
@@ -628,10 +568,10 @@ char* express_number(int value, char c)
 	}
 	else
 	{
-		file_print("Given symbol ", stderr);
+		fputs("Given symbol ", stderr);
 		fputc(c, stderr);
-		file_print(" to express immediate value ", stderr);
-		file_print(numerate_number(value), stderr);
+		fputs(" to express immediate value ", stderr);
+		fputs(int2str(value, 10, TRUE), stderr);
 		fputc('\n', stderr);
 		exit(EXIT_FAILURE);
 	}
@@ -655,17 +595,91 @@ char* express_number(int value, char c)
 	}
 	else
 	{
-		file_print("Got invalid ByteMode in express_number\n", stderr);
+		fputs("Got invalid ByteMode in express_number\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
 	stringify(ch, size, ByteMode, value, shift);
 
-	if(!BigEndian) LittleEndian(ch);
-	else if(!BigBitEndian) reverseBitOrder(ch);
+	if(!BigEndian) LittleEndian(ch, ByteMode);
 	return ch;
 }
 
+char* express_word(int value, char c)
+{
+	char* s = calloc(43, sizeof(char));
+	s[0] = '.';
+	char* ch = s + 1;
+	require(NULL != ch, "Exhausted available memory\n");
+	int size;
+	int shift;
+	int immediate;
+	if('!' == c)
+	{
+		/* Corresponds to RISC-V I format */
+		immediate = (value & 0xFFF) << 20;
+	}
+	else if('@' == c)
+	{
+		/* Corresponds to RISC-V S format */
+		immediate = ((value & 0x1F) << 7) | ((value & 0xFE0) << (31 - 11));
+	}
+	else if('~' == c)
+	{
+		/* Corresponds with RISC-V U format */
+		if ((value & 0xFFF) < 0x800)
+		{
+			immediate = value & 0xFFFFF000;
+		}
+		else
+		{
+			immediate = (value & 0xFFFFF000) + 0x1000;
+		}
+	}
+	else if('%' == c)
+	{
+		/* provides an option for 32bit immediate constants */
+		immediate = value & 0xFFFFFFFF;
+		/* Drop the leading . */
+		ch = s;
+	}
+	else
+	{
+		fputs("Given symbol ", stderr);
+		fputc(c, stderr);
+		fputs(" to express immediate value ", stderr);
+		fputs(int2str(value, 10, TRUE), stderr);
+		fputc('\n', stderr);
+		exit(EXIT_FAILURE);
+	}
+
+	if(HEX == ByteMode)
+	{
+		size = 4 * 2;
+		shift = 4;
+	}
+	else if(OCTAL == ByteMode)
+	{
+		size = 4 * 3;
+		shift = 3;
+	}
+	else if(BINARY == ByteMode)
+	{
+		size = 4 * 8;
+		shift = 1;
+	}
+	else
+	{
+		fputs("Got invalid ByteMode in express_number\n", stderr);
+		exit(EXIT_FAILURE);
+	}
+
+	stringify(ch, size, ByteMode, immediate, shift);
+
+	if(!BigEndian) LittleEndian(ch, ByteMode);
+	return s;
+}
+
 void eval_immediates(struct blob* p)
 {
 	struct blob* i;
@@ -677,11 +691,11 @@ void eval_immediates(struct blob* p)
 		else if('<' == i->Text[0]) continue;
 		else if(NULL == i->Expression)
 		{
-			if((X86 == Architecture) || (AMD64 == Architecture) || (ARMV7L == Architecture) || (AARM64 == Architecture))
+			if((X86 == Architecture) || (AMD64 == Architecture) || (ARMV7L == Architecture) || (AARM64 == Architecture) || (PPC64LE == Architecture))
 			{
 				if(in_set(i->Text[0], "%~@!"))
 				{
-					value = numerate_string(i->Text + 1);
+					value = strtoint(i->Text + 1);
 
 					if(('0' == i->Text[1]) || (0 != value))
 					{
@@ -689,9 +703,21 @@ void eval_immediates(struct blob* p)
 					}
 				}
 			}
+			else if((RISCV32 == Architecture) || (RISCV64 == Architecture))
+			{
+				if(in_set(i->Text[0], "%~@!"))
+				{
+					value = strtoint(i->Text + 1);
+
+					if(('0' == i->Text[1]) || (0 != value))
+					{
+						i->Expression = express_word(value, i->Text[0]);
+					}
+				}
+			}
 			else if(KNIGHT == Architecture)
 			{
-				value = numerate_string(i->Text);
+				value = strtoint(i->Text);
 				if(('0' == i->Text[0]) || (0 != value))
 				{
 					i->Expression = express_number(value, '@');
@@ -699,7 +725,7 @@ void eval_immediates(struct blob* p)
 			}
 			else
 			{
-				file_print("Unknown architecture received in eval_immediates\n", stderr);
+				fputs("Unknown architecture received in eval_immediates\n", stderr);
 				exit(EXIT_FAILURE);
 			}
 		}
@@ -718,15 +744,15 @@ void print_hex(struct Token* p)
 		}
 		else if(NULL != i->contents->Expression)
 		{
-			file_print(i->contents->Expression, destination_file);
+			fputs(i->contents->Expression, destination_file);
 			if(NEWLINE != i->next->contents->type) fputc(' ', destination_file);
 		}
 		else
 		{
 			line_error(i->filename, i->linenumber);
-			file_print("Received invalid other; ", stderr);
-			file_print(i->contents->Text, stderr);
-			file_print("\n", stderr);
+			fputs("Received invalid other; ", stderr);
+			fputs(i->contents->Text, stderr);
+			fputs("\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -738,7 +764,6 @@ int main(int argc, char **argv)
 	BigEndian = TRUE;
 	Architecture = KNIGHT;
 	destination_file = stdout;
-	BigBitEndian = TRUE;
 	ByteMode = HEX;
 	char* filename;
 	char* arch;
@@ -769,12 +794,24 @@ int main(int argc, char **argv)
 		{
 			option_index = option_index + 1;
 		}
-		else if(match(argv[option_index], "--BigEndian") || match(argv[option_index], "--big-endian"))
+		else if(match(argv[option_index], "--BigEndian"))
+		{
+			fputs("--BigEndian is now depreciated and replaced by --big-endian\nAnd will stop working in a future release\n", stderr);
+			BigEndian = TRUE;
+			option_index = option_index + 1;
+		}
+		else if(match(argv[option_index], "--big-endian"))
 		{
 			BigEndian = TRUE;
 			option_index = option_index + 1;
 		}
-		else if(match(argv[option_index], "--LittleEndian") || match(argv[option_index], "--little-endian"))
+		else if(match(argv[option_index], "--LittleEndian"))
+		{
+			fputs("--LittleEndian is now depreciated and replaced by --little-endian\nAnd will stop working in a future release\n", stderr);
+			BigEndian = FALSE;
+			option_index = option_index + 1;
+		}
+		else if(match(argv[option_index], "--little-endian"))
 		{
 			BigEndian = FALSE;
 			option_index = option_index + 1;
@@ -787,11 +824,14 @@ int main(int argc, char **argv)
 			else if(match("amd64", arch)) Architecture = AMD64;
 			else if(match("armv7l", arch)) Architecture = ARMV7L;
 			else if(match("aarch64", arch)) Architecture = AARM64;
+			else if(match("ppc64le", arch)) Architecture = PPC64LE;
+			else if(match("riscv32", arch)) Architecture = RISCV32;
+			else if(match("riscv64", arch)) Architecture = RISCV64;
 			else
 			{
-				file_print("Unknown architecture: ", stderr);
-				file_print(arch, stderr);
-				file_print(" know values are: knight-native, knight-posix, x86, amd64, armv7l and aarch64", stderr);
+				fputs("Unknown architecture: ", stderr);
+				fputs(arch, stderr);
+				fputs(" know values are: knight-native, knight-posix, x86, amd64, armv7l, aarch64, ppc64le, riscv32 and riscv64", stderr);
 				exit(EXIT_FAILURE);
 			}
 			option_index = option_index + 2;
@@ -803,11 +843,11 @@ int main(int argc, char **argv)
 		}
 		else if(match(argv[option_index], "-h") || match(argv[option_index], "--help"))
 		{
-			file_print("Usage: ", stderr);
-			file_print(argv[0], stderr);
-			file_print(" --file FILENAME1 {-f FILENAME2} (--big-endian|--little-endian) ", stderr);
-			file_print("[--architecture name]\nArchitectures: knight-native, knight-posix, x86, amd64 and armv7\n", stderr);
-			file_print("To leverage octal or binary output: --octal, --binary\n", stderr);
+			fputs("Usage: ", stderr);
+			fputs(argv[0], stderr);
+			fputs(" --file FILENAME1 {-f FILENAME2} (--big-endian|--little-endian) ", stderr);
+			fputs("[--architecture name]\nArchitectures: knight-native, knight-posix, x86, amd64, armv7, riscv32 and riscv64\n", stderr);
+			fputs("To leverage octal or binary output: --octal, --binary\n", stderr);
 			exit(EXIT_SUCCESS);
 		}
 		else if(match(argv[option_index], "-f") || match(argv[option_index], "--file"))
@@ -817,9 +857,9 @@ int main(int argc, char **argv)
 
 			if(NULL == source_file)
 			{
-				file_print("The file: ", stderr);
-				file_print(argv[option_index + 1], stderr);
-				file_print(" can not be opened!\n", stderr);
+				fputs("The file: ", stderr);
+				fputs(argv[option_index + 1], stderr);
+				fputs(" can not be opened!\n", stderr);
 				exit(EXIT_FAILURE);
 			}
 
@@ -835,9 +875,9 @@ int main(int argc, char **argv)
 
 			if(NULL == destination_file)
 			{
-				file_print("The file: ", stderr);
-				file_print(argv[option_index + 1], stderr);
-				file_print(" can not be opened!\n", stderr);
+				fputs("The file: ", stderr);
+				fputs(argv[option_index + 1], stderr);
+				fputs(" can not be opened!\n", stderr);
 				exit(EXIT_FAILURE);
 			}
 			option_index = option_index + 2;
@@ -849,19 +889,19 @@ int main(int argc, char **argv)
 		}
 		else if(match(argv[option_index], "-V") || match(argv[option_index], "--version"))
 		{
-			file_print("M1 1.1.0\n", stdout);
+			fputs("M1 1.1.0\n", stdout);
 			exit(EXIT_SUCCESS);
 		}
 		else
 		{
-			file_print("Unknown option\n", stderr);
+			fputs("Unknown option\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 	}
 
 	if(NULL == token_list)
 	{
-		file_print("Either no input files were given or they were empty\n", stderr);
+		fputs("Either no input files were given or they were empty\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
@@ -872,7 +912,6 @@ int main(int argc, char **argv)
 	preserve_other(blob_list);
 	print_hex(token_list);
 
-	fflush(destination_file);
 	fclose(destination_file);
 
 	return EXIT_SUCCESS;
