@@ -17,59 +17,7 @@
  * along with mescc-tools.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <sys/stat.h>
-
-#define max_string 4096
-//CONSTANT max_string 4096
-#define TRUE 1
-//CONSTANT TRUE 1
-#define FALSE 0
-//CONSTANT FALSE 0
-
-// CONSTANT KNIGHT 0
-#define KNIGHT 0
-// CONSTANT X86 1
-#define X86 1
-// CONSTANT AMD64 2
-#define AMD64 2
-// CONSTANT ARMV7L 40
-#define ARMV7L 40
-// CONSTANT AARM64 80
-#define AARM64 80
-
-// CONSTANT HEX 16
-#define HEX 16
-// CONSTANT OCTAL 8
-#define OCTAL 8
-// CONSTANT BINARY 2
-#define BINARY 2
-
-
-/* Imported functions */
-char* numerate_number(int a);
-int in_set(int c, char* s);
-int match(char* a, char* b);
-int numerate_string(char *a);
-void file_print(char* s, FILE* f);
-void require(int bool, char* error);
-
-struct input_files
-{
-	struct input_files* next;
-	char* filename;
-};
-
-struct entry
-{
-	struct entry* next;
-	unsigned target;
-	char* name;
-};
-
+#include "hex2_globals.h"
 
 /* Globals */
 FILE* output;
@@ -85,12 +33,16 @@ char* filename;
 int linenumber;
 int ALIGNED;
 
+/* For processing bytes */
+int hold;
+int toggle;
+
 void line_error()
 {
-	file_print(filename, stderr);
-	file_print(":", stderr);
-	file_print(numerate_number(linenumber), stderr);
-	file_print(" :", stderr);
+	fputs(filename, stderr);
+	fputs(":", stderr);
+	fputs(int2str(linenumber, 10, FALSE), stderr);
+	fputs(" :", stderr);
 }
 
 int consume_token(FILE* source_file)
@@ -168,9 +120,9 @@ unsigned GetTarget(char* c)
 			return i->target;
 		}
 	}
-	file_print("Target label ", stderr);
-	file_print(c, stderr);
-	file_print(" is not valid\n", stderr);
+	fputs("Target label ", stderr);
+	fputs(c, stderr);
+	fputs(" is not valid\n", stderr);
 	exit(EXIT_FAILURE);
 }
 
@@ -202,9 +154,9 @@ void range_check(int displacement, int number_of_bytes)
 	{
 		if((8388607 < displacement) || (displacement < -8388608))
 		{
-			file_print("A displacement of ", stderr);
-			file_print(numerate_number(displacement), stderr);
-			file_print(" does not fit in 3 bytes\n", stderr);
+			fputs("A displacement of ", stderr);
+			fputs(int2str(displacement, 10, TRUE), stderr);
+			fputs(" does not fit in 3 bytes\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 		return;
@@ -213,9 +165,9 @@ void range_check(int displacement, int number_of_bytes)
 	{
 		if((32767 < displacement) || (displacement < -32768))
 		{
-			file_print("A displacement of ", stderr);
-			file_print(numerate_number(displacement), stderr);
-			file_print(" does not fit in 2 bytes\n", stderr);
+			fputs("A displacement of ", stderr);
+			fputs(int2str(displacement, 10, TRUE), stderr);
+			fputs(" does not fit in 2 bytes\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 		return;
@@ -224,15 +176,15 @@ void range_check(int displacement, int number_of_bytes)
 	{
 		if((127 < displacement) || (displacement < -128))
 		{
-			file_print("A displacement of ", stderr);
-			file_print(numerate_number(displacement), stderr);
-			file_print(" does not fit in 1 byte\n", stderr);
+			fputs("A displacement of ", stderr);
+			fputs(int2str(displacement, 10, TRUE), stderr);
+			fputs(" does not fit in 1 byte\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 		return;
 	}
 
-	file_print("Invalid number of bytes given\n", stderr);
+	fputs("Invalid number of bytes given\n", stderr);
 	exit(EXIT_FAILURE);
 }
 
@@ -275,9 +227,9 @@ int Architectural_displacement(int target, int base)
 		if (target & 3)
 		{
 			line_error();
-			file_print("error: Unaligned branch target: ", stderr);
-			file_print(scratch, stderr);
-			file_print(", aborting\n", stderr);
+			fputs("error: Unaligned branch target: ", stderr);
+			fputs(scratch, stderr);
+			fputs(", aborting\n", stderr);
 			exit(EXIT_FAILURE);
 		}
 		/*
@@ -309,8 +261,20 @@ int Architectural_displacement(int target, int base)
 	{
 		return ((target - base) - 8 + (3 & base));
 	}
+	else if(ALIGNED && (PPC64LE == Architecture))
+	{
+		ALIGNED = FALSE;
+		/* set Link register with branch */
+		return (target - (base & 0xFFFFFFFC )) | 1;
+	}
+	else if(PPC64LE == Architecture)
+	{
+		/* DO *NOT* set link register with branch */
+		return (target - (base & 0xFFFFFFFC));
+	}
+	else if(RISCV32 == Architecture || RISCV64 == Architecture) return (target - base);
 
-	file_print("Unknown Architecture, aborting before harm is done\n", stderr);
+	fputs("Unknown Architecture, aborting before harm is done\n", stderr);
 	exit(EXIT_FAILURE);
 }
 
@@ -324,7 +288,7 @@ void Update_Pointer(char ch)
 	else
 	{
 		line_error();
-		file_print("storePointer given unknown\n", stderr);
+		fputs("storePointer given unknown\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -367,9 +331,9 @@ void storePointer(char ch, FILE* source_file)
 	else
 	{
 		line_error();
-		file_print("error: storePointer reached impossible case: ch=", stderr);
+		fputs("error: storePointer reached impossible case: ch=", stderr);
 		fputc(ch, stderr);
-		file_print("\n", stderr);
+		fputs("\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -411,9 +375,6 @@ int binary(int c, FILE* source_file)
 	return -1;
 }
 
-
-int hold;
-int toggle;
 void process_byte(char c, FILE* source_file, int write)
 {
 	if(HEX == ByteMode)
@@ -478,7 +439,7 @@ void process_byte(char c, FILE* source_file, int write)
 
 void pad_to_align(int write)
 {
-	if((ARMV7L == Architecture) || (AARM64 == Architecture))
+	if((ARMV7L == Architecture) || (AARM64 == Architecture) || (RISCV32 == Architecture) || (RISCV64 == Architecture))
 	{
 		if(1 == (ip & 0x1))
 		{
@@ -507,9 +468,9 @@ void first_pass(struct input_files* input)
 
 	if(NULL == source_file)
 	{
-		file_print("The file: ", stderr);
-		file_print(input->filename, stderr);
-		file_print(" can not be opened!\n", stderr);
+		fputs("The file: ", stderr);
+		fputs(input->filename, stderr);
+		fputs(" can not be opened!\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
@@ -558,9 +519,9 @@ void second_pass(struct input_files* input)
 	/* Something that should never happen */
 	if(NULL == source_file)
 	{
-		file_print("The file: ", stderr);
-		file_print(input->filename, stderr);
-		file_print(" can not be opened!\nWTF-pass2\n", stderr);
+		fputs("The file: ", stderr);
+		fputs(input->filename, stderr);
+		fputs(" can not be opened!\nWTF-pass2\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
@@ -579,171 +540,3 @@ void second_pass(struct input_files* input)
 
 	fclose(source_file);
 }
-
-/* Standard C main program */
-int main(int argc, char **argv)
-{
-	ALIGNED = FALSE;
-	BigEndian = TRUE;
-	jump_tables = calloc(65537, sizeof(struct entry*));
-
-	Architecture = KNIGHT;
-	Base_Address = 0;
-	struct input_files* input = NULL;
-	output = stdout;
-	char* output_file = "";
-	exec_enable = TRUE;
-	ByteMode = HEX;
-	scratch = calloc(max_string + 1, sizeof(char));
-	char* arch;
-	struct input_files* temp;
-
-	int option_index = 1;
-	while(option_index <= argc)
-	{
-		if(NULL == argv[option_index])
-		{
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "--BigEndian") || match(argv[option_index], "--big-endian"))
-		{
-			BigEndian = TRUE;
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "--LittleEndian") || match(argv[option_index], "--little-endian"))
-		{
-			BigEndian = FALSE;
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "--exec_enable"))
-		{
-			/* Effectively a NOP */
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "--non-executable"))
-		{
-			exec_enable = FALSE;
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "-A") || match(argv[option_index], "--architecture"))
-		{
-			arch = argv[option_index + 1];
-			if(match("knight-native", arch) || match("knight-posix", arch)) Architecture = KNIGHT;
-			else if(match("x86", arch)) Architecture = X86;
-			else if(match("amd64", arch)) Architecture = AMD64;
-			else if(match("armv7l", arch)) Architecture = ARMV7L;
-			else if(match("aarch64", arch)) Architecture = AARM64;
-			else
-			{
-				file_print("Unknown architecture: ", stderr);
-				file_print(arch, stderr);
-				file_print(" know values are: knight-native, knight-posix, x86, amd64 and armv7l", stderr);
-			}
-			option_index = option_index + 2;
-		}
-		else if(match(argv[option_index], "-b") || match(argv[option_index], "--binary"))
-		{
-			ByteMode = BINARY;
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "-B") || match(argv[option_index], "--BaseAddress") || match(argv[option_index], "--base-address"))
-		{
-			Base_Address = numerate_string(argv[option_index + 1]);
-			option_index = option_index + 2;
-		}
-		else if(match(argv[option_index], "-h") || match(argv[option_index], "--help"))
-		{
-			file_print("Usage: ", stderr);
-			file_print(argv[0], stderr);
-			file_print(" --file FILENAME1 {-f FILENAME2} (--big-endian|--little-endian)", stderr);
-			file_print(" [--base-address 0x12345] [--architecture name]\nArchitecture:", stderr);
-			file_print(" knight-native, knight-posix, x86, amd64, armv7l and aarch64\n", stderr);
-			file_print("To leverage octal or binary input: --octal, --binary\n", stderr);
-			exit(EXIT_SUCCESS);
-		}
-		else if(match(argv[option_index], "-f") || match(argv[option_index], "--file"))
-		{
-			temp = calloc(1, sizeof(struct input_files));
-			temp->filename = argv[option_index + 1];
-			temp->next = input;
-			input = temp;
-			option_index = option_index + 2;
-		}
-		else if(match(argv[option_index], "-o") || match(argv[option_index], "--output"))
-		{
-			output_file = argv[option_index + 1];
-			output = fopen(output_file, "w");
-
-			if(NULL == output)
-			{
-				file_print("The file: ", stderr);
-				file_print(argv[option_index + 1], stderr);
-				file_print(" can not be opened!\n", stderr);
-				exit(EXIT_FAILURE);
-			}
-			option_index = option_index + 2;
-		}
-		else if(match(argv[option_index], "-O") || match(argv[option_index], "--octal"))
-		{
-			ByteMode = OCTAL;
-			option_index = option_index + 1;
-		}
-		else if(match(argv[option_index], "-V") || match(argv[option_index], "--version"))
-		{
-			file_print("hex2 1.1.0\n", stdout);
-			exit(EXIT_SUCCESS);
-		}
-		else
-		{
-			file_print("Unknown option\n", stderr);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	/* Catch a common mistake */
-	if((KNIGHT != Architecture) && (0 == Base_Address))
-	{
-		file_print(">> WARNING <<\n>> WARNING <<\n>> WARNING <<\n", stderr);
-		file_print("If you are not generating a ROM image this binary will likely not work\n", stderr);
-	}
-
-	/* Catch implicitly false assumptions */
-	if(BigEndian && ((X86 == Architecture) || ( AMD64 == Architecture) || (ARMV7L == Architecture) || (AARM64 == Architecture)))
-	{
-		file_print(">> WARNING <<\n>> WARNING <<\n>> WARNING <<\n", stderr);
-		file_print("You have specified big endian output on likely a little endian processor\n", stderr);
-		file_print("if this is a mistake please pass --little-endian next time\n", stderr);
-	}
-
-	/* Make sure we have a program tape to run */
-	if (NULL == input)
-	{
-		return EXIT_FAILURE;
-	}
-
-	/* Get all of the labels */
-	ip = Base_Address;
-	first_pass(input);
-
-	/* Fix all the references*/
-	ip = Base_Address;
-	second_pass(input);
-
-	/* flush all writes */
-	fflush(output);
-
-	/* Set file as executable */
-	if(exec_enable && (output != stdout))
-	{
-		/* Close output file */
-		fclose(output);
-
-		if(0 != chmod(output_file, 0750))
-		{
-			file_print("Unable to change permissions\n", stderr);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	return EXIT_SUCCESS;
-}
