@@ -1,21 +1,21 @@
 #! /bin/sh
 ## Copyright (C) 2017 Jeremiah Orians
-## This file is part of stage0.
+## This file is part of mescc-tools.
 ##
-## stage0 is free software: you can redistribute it and/or modify
+## mescc-tools is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
 ##
-## stage0 is distributed in the hope that it will be useful,
+## mescc-tools is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with stage0.  If not, see <http://www.gnu.org/licenses/>.
+## along with mescc-tools.  If not, see <http://www.gnu.org/licenses/>.
 
 set -ex
-./bin/M1 -f test/test5/exec_enable_amd64.M1 --LittleEndian --architecture amd64 -o test/test5/hold
-./bin/hex2 -f elf_headers/elf64.hex2 -f test/test5/hold --LittleEndian --architecture amd64 --BaseAddress 0x00600000 -o test/results/test5-binary --exec_enable
+./bin/M1 -f test/test5/exec_enable_amd64.M1 --little-endian --architecture amd64 -o test/test5/hold
+./bin/hex2 -f elf_headers/elf64.hex2 -f test/test5/hold --little-endian --architecture amd64 --base-address 0x00600000 -o test/results/test5-binary
 exit 0
