@@ -1,21 +1,21 @@
 #! /bin/sh
 ## Copyright (C) 2017 Jeremiah Orians
-## This file is part of stage0.
+## This file is part of mescc-tools.
 ##
-## stage0 is free software: you can redistribute it and/or modify
+## mescc-tools is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
 ## the Free Software Foundation, either version 3 of the License, or
 ## (at your option) any later version.
 ##
-## stage0 is distributed in the hope that it will be useful,
+## mescc-tools is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with stage0.  If not, see <http://www.gnu.org/licenses/>.
+## along with mescc-tools.  If not, see <http://www.gnu.org/licenses/>.
 
 set -ex
-./bin/M1 -f test/test6/exec_enable_i386.M1 --LittleEndian --architecture x86 -o test/test6/hold
-./bin/hex2 -f elf_headers/elf32.hex2 -f test/test6/hold --LittleEndian --architecture x86 --BaseAddress 0x8048000 -o test/results/test6-binary --exec_enable
+./bin/M1 -f test/test6/exec_enable_i386.M1 --little-endian --architecture x86 -o test/test6/hold
+./bin/hex2 -f elf_headers/elf32.hex2 -f test/test6/hold --little-endian --architecture x86 --base-address 0x8048000 -o test/results/test6-binary
 exit 0
